#############################################################################################
##
## From Fang Yu (UNMC), Ming-Hui Chen(UCONN) and Lynn Kuo (UCONN)
## This is a trial version of R code for idnetifying DE genes from RNA-Seq data
## Two Groups will be under comparison
## Avaiable from May 14 2015
##
## Input: 
## 	Datause:   The RNA-Seq dataset, which is organized with each row reprenting a gene 
##	    	   and each column representing a replicate
##		   We assume that "datause" follows negative binomial ditribution
## 	nburnin:   The iteration for burn-in
## 	nmcmc:     The nubmer of interation used to obtain posterior estiamtion after burnin
## 	design:    A categorical vector with 1,2 to indicate the group under comaprison
##	    	   For example, an experiment with three replicates per group will have
##		   design=c(1,1,1,2,2,2)
##	prior_p:   Gamma Prior parameter for the probability p		
##	prior_phi: Inverse-Gamma prior parameter for the overdispersion parameter phi
##	pcut: 	   The targeted false discovery rate level 	
##	
## Output:
##	order:	   The ordering the genes in a descending support of differential gene expression
##	post_prob: The posterior esimator on the gene expression levels 
##	gammau:	   The calculated gamma value 
##	nDE:	   Total number of idnetified DE genes 
##	DE_sign:   Indicator for identified DE genes (1: DE gene; 0: EE gene)
##
#############################################################################################

CCC.RNASeq=function(datause,nburnin,nmcmc,design,prior_p,prior_phi,pcut)
{
dyn.load("gamcal1u.dll")
dyn.load("gamcal2u.dll")
dyn.load("fdrcalubb.dll")

	mj=as.integer(length(unique(design)))
	ngene=nrow(datause)
	nrept=ncol(datause)

	nrepv=NULL
	for(i in 1:mj)
	{
		nrepv=c(nrepv,sum(design==i))
	}
	ndatatsum1=apply(datause,2,median)

	if(! is.integer(datause)) {
       print("The read-in data is not integer!") 
       return()
       }

asrpp0=prior_p[1]
bsrpp0=prior_p[2]
aphi0=prior_phi[1]
bphi0=prior_phi[2]

dd1=.Fortran("gamcal1",iseed=as.integer(1234),inins=as.integer(1),
nburnin=as.integer(nburnin),nsimur=as.integer(nmcmc),
n=as.integer(ngene),mj=as.integer(mj),mll=as.integer(nrepv),
ml=as.integer(nrept),ndata=datause,ndatatsum1=as.integer(ndatatsum1),
asrpp0=as.double(asrpp0),bsrpp0=as.double(bsrpp0),
aphi0=as.double(aphi0),bphi0=as.double(bphi0),
rmeanrpp1=matrix(0.0,ngene,2),rmeanphi1=as.double(c(1.0,1.0)),
rlogrppd=as.double(matrix(0.0,ngene,1)),
rlogsqrppd=as.double(matrix(0.0,ngene,1)),gammau=as.double(matrix(0.0,ngene,1)))

dd2=.Fortran('gamcal2',iseed=as.integer(1234),
nburnin=as.integer(nburnin),nsimur=as.integer(nmcmc),
n=as.integer(ngene),mj=as.integer(mj),mll=as.integer(nrepv),
ml=as.integer(nrept),ndata=datause,ndatatsum1=as.integer(ndatatsum1),
asrpp0=as.double(asrpp0),bsrpp0=as.double(bsrpp0),
aphi0=as.double(aphi0),bphi0=as.double(bphi0),
rmeanrpp2=as.double(matrix(0.0,ngene,1)),rmeanphi2=as.double(1.0))


nstep=1
jju=1
gammau=dd1$gammau

fdrresult=.Fortran("fdrcal",iseed=as.integer(1234),nburnin=as.integer(100),nsimur=as.integer(100),
pcut=as.double(pcut),n=as.integer(ngene),mj=as.integer(mj),ml=as.integer(nrept),mll=as.integer(nrepv),
asrpp0=as.double(asrpp0),bsrpp0=as.double(bsrpp0),aphi0=as.double(aphi0),bphi0=as.double(bphi0),
ngee=as.integer(ngene*0.1),gammau=as.double(gammau), jju=as.integer(jju),nstartp=as.integer(nstartp),
ndatatsum1=as.integer(ndatatsum1),rmeanrpp1=dd1$rmeanrpp1,rmeanrpp2=as.double(dd2$rmeanrpp2),
rmeanphi1=as.double(dd1$rmeanphi1),rmeanphi2=as.double(dd2$rmeanphi2), fdrf=as.double(fdrf),
gcut=as.double(0.75),nstep=as.integer(nstep))

nstep=0
jju=as.integer(fdrresult$nstartp/20+1)*20

fdrresult=.Fortran("fdrcal",iseed=as.integer(1234),nburnin=as.integer(1000),nsimur=as.integer(nmcmc),
pcut=as.double(pcut),n=as.integer(ngene),mj=as.integer(mj),ml=as.integer(nrept),mll=as.integer(nrepv),
asrpp0=as.double(asrpp0),bsrpp0=as.double(bsrpp0),aphi0=as.double(aphi0),bphi0=as.double(bphi0),
ngee=as.integer(ngene*0.1),gammau=as.double(gammau), jju=as.integer(jju),nstartp=as.integer(nstartp),
ndatatsum1=as.integer(ndatatsum1),rmeanrpp1=dd1$rmeanrpp1,rmeanrpp2=as.double(dd2$rmeanrpp2),
rmeanphi1=as.double(dd1$rmeanphi1),rmeanphi2=as.double(dd2$rmeanphi2), fdrf=as.double(fdrf),
gcut=as.double(0.75),nstep=as.integer(nstep))

nselect=fdrresult$nstartp
ids=order(gammau,decreasing=T)
DE_Y=matrix(0,ngene,1)
DE_Y[ids[1:nselect]]=1

list("order"=ids,"post_prob"=dd1$rmeanrpp1,"gammau"=gammau,"nDE"=nselect,"DE_sign"=DE_Y)
}





